/*
 * This file is part of the Polkit-tqt project
 * Copyright (C) 2009 Dario Freddi <drf@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB. If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef POLKIT_TQT_GUI_ACTIONBUTTON_P_H
#define POLKIT_TQT_GUI_ACTIONBUTTON_P_H

#include <polkit-tqt-gui-actionbutton.h>

class TQButton;
template <typename> class TQValueList;


namespace PolkitTQt
{

namespace Gui
{

class ActionButtonPrivate
{
  public:
    ActionButtonPrivate()
    {
    }

    virtual ~ActionButtonPrivate()
    {
    }

    void addButton(TQButton *button);
    void removeButton(TQButton *button);

    ActionButton *q;

    TQValueList<TQButton*> buttons;
};

}

}

#endif
