#define POLKIT_AGENT_I_KNOW_API_IS_SUBJECT_TO_CHANGE 1

#include <polkitagent/polkitagent.h>

#include "pkagentexample.h"
#include <unistd.h>
#include <tqsessionmanager.h>

#include "polkit-tqt-subject.h"

PkAgentExample::PkAgentExample(int argc, char **argv) : TQApplication(argc, argv)
{
  PolkitTQt::UnixSessionSubject session(getpid());
  m_listener.registerListener(session, "/org/tqt/PolicyKit1/AuthenticationAgent");
}

void PkAgentExample::commitData(TQSessionManager &sm)
{
  sm.setRestartHint(TQSessionManager::RestartNever);
}

#include "pkagentexample.moc"
