// This is an example not a library
/***************************************************************************
 *   Copyright (C) 2008 Daniel Nicoletti <dantti85-pk@yahoo.com.br>        *
 *   Copyright (C) 2009 Radek Novacek    <rnovacek@redhat.com>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef PKEXAMPLE_H
#define PKEXAMPLE_H

#include <tqvariant.h>
#include <tqmainwindow.h>

class TQVBoxLayout;
class TQGridLayout;
class TQSpacerItem;
class TQToolBar;
class TQPopupMenu;
class TQLabel;
class TQPushButton;
class TQComboBox;
class TQListView;

class PkExample : public TQMainWindow
{
    TQ_OBJECT

public:
    PkExample();

    TQLabel *kickL;
    TQPushButton *kickPB;
    TQLabel *cryL;
    TQPushButton *cryPB;
    TQLabel *bleedL;
    TQPushButton *bleedPB;
    TQLabel *playL;
    TQPushButton *playPB;
    TQLabel *listenL;
    TQPushButton *listenPB;
    TQLabel *deleteL;
    TQPushButton *deletePB;
    TQLabel *deleteL_2;
    TQComboBox *setCB;
    TQPushButton *setPB;
    TQLabel *deleteL_3;
    TQPushButton *shoutPB;
    TQListView *actionList;
    TQMenuBar *menubar;
    TQPopupMenu *menuActions;
    TQToolBar *toolBar;

  protected:
    TQVBoxLayout *vLayout;
    TQGridLayout *grid;

  protected slots:
    virtual void languageChange();

  private slots:
    void activateAction();
    void actionActivated();

};

#endif
